package com.xirihanlin.compass;


import android.content.Context;
import android.content.res.Resources;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.View;

public class CompassView extends View{

	private Paint textPaint;
	private Paint markingPaint;
	private Paint circlePaint;
	private String northString;
	private String eastString;
	private String southString;
	private String westString;
	private int textHeight;
	
	public CompassView(Context context) {
		super(context);
		initCompassView();
	}
	
	public CompassView(Context context, AttributeSet attrs)
	{
		super(context, attrs);
		initCompassView();
	}
	
	public CompassView(Context context, AttributeSet attrs, int defaultStyle)
	{
		super(context, attrs, defaultStyle);
		initCompassView();
	}
	
	protected void initCompassView() {
		setFocusable(true);
		
		Resources resources = getResources();
		
		circlePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		circlePaint.setColor(resources.getColor(R.color.background_color));
		circlePaint.setStrokeWidth(1);
		circlePaint.setStyle(Paint.Style.FILL_AND_STROKE);
		
		
		northString = resources.getString(R.string.north);
		eastString = resources.getString(R.string.east);
		southString = resources.getString(R.string.south);
		westString = resources.getString(R.string.west);
		
		textPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		textPaint.setColor(resources.getColor(R.color.text_color));
		
		textHeight = (int)textPaint.measureText("yY");
		
		markingPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		markingPaint.setColor(resources.getColor(R.color.marking_color));
	}

	@Override
	protected void onDraw(Canvas canvas) {
		int px = getMeasuredWidth() / 2;
		int py = getMeasuredHeight() / 2;
		int radius = Math.min(px, py);
		
		canvas.drawCircle(px, py, radius, circlePaint);
		
		canvas.save();
		canvas.rotate(-_bearing, px, py);
		
		int textWidth = (int)textPaint.measureText("W");
		int cardinalX = px - textWidth / 2;
		int cardinalY = py - radius + textHeight;
		
		for(int i = 0; i < 24; i++)
		{
			canvas.drawLine(px, py - radius, px, py - radius + 10, markingPaint);
			canvas.save();
			canvas.translate(0, textHeight);
			
			if(i % 6 == 0)
			{
				String dirString = "";
				switch (i) {
				case 0:
				{
					dirString = northString;
					canvas.drawLine(px, 2 * textHeight, px - 5, 3 * textHeight, markingPaint);
					canvas.drawLine(px, 2 * textHeight, px + 5, 3 * textHeight, markingPaint);
					break;
				}
				case 6:
					dirString = eastString;
					break;
				case 12:
					dirString = southString;
					break;
				case 18:
					dirString = westString;
					break;
				default:
					break;
				}
				canvas.drawText(dirString, cardinalX, cardinalY, textPaint);
			}
			else if (i % 3 == 0) 
			{
				String angleString = String.valueOf(i * 15);
				int angleTextWidth = (int)textPaint.measureText(angleString);
				int angleX = (int)(px - angleTextWidth / 2);
				int angleY = py - radius + textHeight;
				canvas.drawText(angleString, angleX, angleY, textPaint);
			}
			canvas.restore();
			canvas.rotate(15, px, py);
		}
		canvas.restore();
	}

	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		int measureWidth = measure(widthMeasureSpec);
		int measureHeight = measure(heightMeasureSpec);
		int d = Math.min(measureWidth, measureHeight);
		setMeasuredDimension(d, d);
	}
	
	private int measure(int measureSpec) {
		int result = 0;
		int specMode = MeasureSpec.getMode(measureSpec);
		int specSize = MeasureSpec.getSize(measureSpec);
		if (specMode == MeasureSpec.UNSPECIFIED) {
			result = 200;
		}
		else {
			result = specSize;
		}
		return result;
	}
	
	private float _bearing = 45;
	public void setBearing(int bearing) {
		_bearing = bearing;
	}
	public float getBearing() {
		return _bearing;
	}
}
